/* ------------------------- fftot.c ---------------------------------- */
/*									*/
/* Author:	Eyal Lebedinsky						*/
/* Date:	Jan 1994						*/
/* Version:	20 December 1995					*/
/*									*/
/* called from fftg.c and generates a control table.			*/
/*									*/
/* This program is released into the public domain.			*/
/*									*/
/*----------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>

static int	stmts = 0, file = 0, flen = 0, tlen = 0;
static char	fname[256] = "", tname[256] = "";

void
start_fft (file_name, ep_name)
char	*file_name, *ep_name;
{
	if (file_name) {
		strcpy (tname, ep_name);
		strcat (tname, "aa");
		tlen = strlen (tname);
		strcpy (fname, file_name);
		strcat (fname, "aa.c");
		flen = strlen (fname);
		file = 0;
	}

	freopen (fname, "wt", stdout);
	printf ("short t_%s[] = {\n", tname);
	stmts = 0;
}

void
end_fft (file_name, ep_name)
char	*file_name, *ep_name;
{
	int	i;

	printf ("0, 0};\n");

	printf ("#define FFT_NAME   %s\n", ep_name);
	printf ("#define TABLE_NAME t_%s\n", ep_name);

	strcpy (tname, ep_name);
	strcat (tname, "aa");
	for (i = 0; i <= file; ++i) {
		printf ("extern short t_%s[];\n", tname, i);
		if (++tname[tlen - 1] > 'z') {
			tname[tlen - 1] = 'a';
			++tname[tlen - 2];
		}
	}

	printf ("static short *TABLE_NAME[] = {\n");
	strcpy (tname, ep_name);
	strcat (tname, "aa");
	for (i = 0; i <= file; ++i) {
		printf ("\tt_%s,\n", tname, i);
		if (++tname[tlen - 1] > 'z') {
			tname[tlen - 1] = 'a';
			++tname[tlen - 2];
		}
	}
	printf ("0};\n");

	printf ("#include \"fftst.c\"\n");
	fclose (stdout);
}

void
break_fft (n)
int	n;
{
	stmts += n;
	if (stmts > 2000) {
		if (++fname[flen-3] > 'z') {		/* next file */
			fname[flen-3] = 'a';
			++fname[flen-4];
		}
		++file;					/* next table */
		if (++tname[tlen - 1] > 'z') {
			tname[tlen - 1] = 'a';
			++tname[tlen - 2];
		}
		printf ("0, %u};\n", file);

		start_fft (NULL, NULL);
	}
}

void
fft1 (i1)
int	i1;
{
	printf ("1,%d,\n", 2*i1);
	break_fft (2);
}

void
fft2 (i1, i2)
int	i1, i2;
{
	printf ("2,%d,%d,\n", 2*i1, 2*i2);
	break_fft (3);
}

void
fft3 (i1, i3, i4)
int	i1, i3, i4;
{
	printf ("3,%d,%d,%d,\n", 2*i1, 2*i3, 2*i4);
	break_fft (4);
}

void
fft4 (i1, i2, i3, i4)
int	i1, i2, i3, i4;
{
	printf ("4,%d,%d,%d,%d,\n", 2*i1, 2*i2, 2*i3, 2*i4);
	break_fft (5);
}

void
fft5 (i1, i2, i3, i4, i5, i6, i7, i8, sm1, sp1, cc1, sm3, sp3, cc3, ind)
int	i1, i2, i3, i4, i5, i6, i7, i8, sm1, sp1, cc1, sm3, sp3, cc3, ind;
{
	printf ("5,%d,%d,%d,%d,%d,%d,%d,%d,",
		2*i1, 2*i2, 2*i3, 2*i4, 2*i5, 2*i6, 2*i7, 2*i8);
	printf ("0x%04x,0x%04x,0x%04x,0x%04x,0x%04x,0x%04x,%d,\n",
		(int)(unsigned short)sm1, (int)(unsigned short)sp1,
		(int)(unsigned short)cc1, (int)(unsigned short)sm3,
		(int)(unsigned short)sp3, (int)(unsigned short)cc3, ind);
	break_fft (16);
}

void
fft7 (i1, i2)
int	i1, i2;
{
	printf ("7,%d,%d,\n", 2*i1, 2*i2);
	break_fft (3);
}

void
fft8 (i1, i2, i3)
int	i1, i2, i3;
{
	printf ("8,%d,%d,%d,\n", 2*i1, 2*i2, 2*i3);
	break_fft (4);
}
